/****************** <lalVerbatim file="BlockHV">
Author: Tibbits, M. M.
$Id: Block.h,v 1.2 2002/07/05 20:45:18 tibbits Exp $
********************************* </lalVerbatim> */

/******************************* <lalLaTeX file="BlockH">
\section{Header \texttt{Block.h}}
\label{s:Block.h}

Provides structures and definitions global to the block search,
notably \texttt{BlockSearchParams}.

\subsection*{Error conditions}

There are no error conditions associated with this header file.

\subsection*{Structures and Unions}

*************************************************** </lalLaTeX> */

#ifndef _BLOCK_H_
#define _BLOCK_H_

#include <math.h>
#include <mpi.h>
#include <lal/LALRCSID.h>
#include <LALWrapperInterface.h>
#include <lal/LALStdlib.h>
#include <lal/AVFactories.h>
#include <lal/LALConstants.h>
#include <lal/LALMoment.h>
#include <lal/Comm.h>
#include "Matrix.h"

#ifdef  __cplusplus
extern "C" {
#endif

NRCSID( BLOCKH, "$Id: Block.h,v 1.2 2002/07/05 20:45:18 tibbits Exp $");

/******************************* <lalLaTeX file="BlockH">

\subsubsection*{Union \texttt{ExchType}}
\index{\texttt{ExchType}}

\noindent
The enumerations are:\\
\\
PassData\\
GetData\\
PassResult\\
RePass\\
ExchFinished\\

*************************************************** </lalLaTeX> */

/*  Identifiers for MPI messages types  */
enum
{
	PassData,
	GetData,
	PassResult,
	RePass,
	ExchFinished
}
ExchType;

/******************************* <lalLaTeX file="BlockH">

\subsubsection*{Structure \texttt{BlockSearchParams}}
\index{\texttt{BlockSearchParams}}

\noindent
The fields are:\\
\\
REAL8VectorVector	*dataRho3\\
REAL8VectorVector	*data\\
gpsTimeInterval		times\\
REAL8Vector		*vec\\
REAL8Vector		*value\\
REAL8Vector		*res\\
INT4Vector		*ndx\\
dataBase		*resultsToStore\\
dataBase		*prevResult\\
REAL8			threshold\\
UINT4			numberOfSequences\\
UINT4			segmentsRemaining\\
UINT4			segmentLength\\
UINT4			numSlaves\\
UINT4			numOutput\\
INT4			reprocessWithRho3\\
INT4			nextVectorToSend\\
INT4			maxVectorLength\\
INT4			searchMaster\\
INT4			vectorId\\
INT4			rank\\
INT4			run\\
CHAR			channel[dbNameLimit]\\
BOOLEAN			initializedRho3\\
*************************************************** </lalLaTeX> */

/*  This datatype is to allow for a vector seqeunce that contains vectors of different lengths  */

typedef struct
tagREAL8VectorVector
{
	REAL8Vector		**vector;
	INT4			numberOfVectors;
}
REAL8VectorVector;

typedef struct
tagBlockSearchParams
{
	REAL8VectorVector	*dataRho3;		/*  Container for data to be reprocessed in Rho3  */
	REAL8VectorVector	*data;			/*  Container for data received from dataconAPI  */
	gpsTimeInterval		times;			/*  The GPS start and end time of the data   */
	REAL8Vector		*vec;			/*  Container for vector received from SearchMaster  */
	REAL8Vector		*value;			/*  Container for sending/receiving Max Value of Data Sequence  */
	REAL8Vector		*res;			/*  Container for sending/receiving Rho2 results  */
	INT4Vector		*ndx;			/*  Container for sending/receiving index of the max value  */

	dataBase		*resultsToStore;	/*  Pointer to hold results to push to the database  */
	dataBase		*prevResult;		/*  Point to end of linked list  */

	REAL8			threshold;		/*  Threshold value for recording an event  */

	UINT4			numberOfSequences;	/*  Number of Sequences in data  */
	UINT4			segmentsRemaining;	/*  Number of Segments left to receive  */
	UINT4			segmentLength;		/*  Length of each vector  */
	UINT4			numSlaves;		/*  Number of friends in Comm World  */
	UINT4			numOutput;		/*  Stores number of points in params->resultsToStore  */

	INT4			reprocessWithRho3;	/*  Boolean, To stop or not to stop?  */
	INT4			nextVectorToSend;	/*  Index of next Vector to send  */
	INT4			maxVectorLength;	/*  Parameter to tell the search to chop segments  */
	INT4			searchMaster;		/*  Boolean, am I in charge?  */
	INT4			vectorId;		/*  Index of Current Vector being processed (on Slave nodes)  */
	INT4			rank;			/*  node number in Comm World  */
	INT4			run;			/*  Stores the iteration of Rho3  */

	CHAR			channel[dbNameLimit];	/*  Origin of Data ~ Hard coded to H2  */

	BOOLEAN			initializedRho3;	/*  Flag set if BlockReparseData finished */

}
BlockSearchParams;

#ifdef  __cplusplus
}
#endif /* C++ protection */

/******************************* <lalLaTeX file="BlockH">
  
\vfill{\footnotesize\input{BlockHV}}

*************************************************** </lalLaTeX> */

#endif /* _BLOCK_H */
