/* -*- mode: c; c-basic-offset: 2; -*- */
/******** <lalVerbatim file="SlopeDatatypesHV"> ********
Author: Daw, E
$Id: SlopeDatatypes.h,v 1.2 2002/07/15 22:03:26 edaw Exp $
********* </lalVerbatim> ********/

#ifndef _SLOPEDATATYPESH
#define _SLOPEDATATYPESH

#include <lal/LALRCSID.h>
#include <LALWrapperInterface.h>

NRCSID( SLOPEDATATYPESH, "$Id: ");

/******** <lalErrTable file="SlopeHErrTab"> */

/*#define SLDEBUG*/

#define SLOPE_DEFAULT_CHANNEL "ifodmro::sequence:primary"
#define SLOPE_DEFAULT_SAMPLING_RATE 16384.0
#define SLOPE_DEFAULT_THRESHOLD 100.0
#define SLOPE_DEFAULT_MEASURE_WIDTH 64      /*number of bins used to*/
                                            /*estimate the slope*/
#define SLOPE_DEFAULT_CORRELATED_WIDTH 300  /*number of adjacent bins*/
                                            /*counted as one peak*/
                                            /*in the peak search */
#define SLOPE_DEFAULT_IFO "H2"

#define INITSEARCH_BADINPUTARGS        1
#define CONDITIONDATA_NONTSDATA        2
#define CONDITIONDATA_DATATYPEUNSUP    3
#define CONDITIONDATA_NOSUCHCHANNEL    4
#define APPLYSEARCH_NOSLAVES           5
#define APPLYSEARCH_NOSUCHFILTER       6
#define CONDITIONDATA_INVALIDFILTHIST  7
#define CONDITIONDATA_TIMEARITHERROR   8

#define INITSEARCH_BADINPUTARGSMESG "Bad input argument list"
#define CONDITIONDATA_NONTSDATAMESG "Non timeseries data"
#define CONDITIONDATA_DATATYPEUNSUPMESG "Data type unsupported"
#define CONDITIONDATA_NOSUCHCHANNELMESG "Channel not found in data"
#define APPLYSEARCH_NOSLAVESMESG "Slaves not supported"
#define APPLYSEARCH_NOSUCHFILTERMESG "This filter type not supported"
#define CONDITIONDATA_INVALIDFILTHISTMESG "History set bit invalid"
#define CONDITIONDATA_TIMEARITHERRORMESG "Inappropriate GPS time arithmetic"

/* these filter types are also defined in lal include file   */
/* .../packages/slopefilters/include/SlopeDetectorFilter.h   */
/* except for FILTER_OUTPUT_PRELIM which is the 'old' slope  */
/* detector filter                                           */

#define FILTER_OUTPUT_PRELIM               10
#define FILTER_OUTPUT_OFFSET               1
#define FILTER_OUTPUT_SLOPE                2
#define FILTER_OUTPUT_ALF                  3
#define FILTER_OUTPUT_TAPS_SET_BOXCAR      4
#define FILTER_OUTPUT_TAPS_SET_GAUSSIAN    5
#define FILTER_OUTPUT_TAPS_SET_SINE        6
#define FILTER_OUTPUT_TAPS_SET_USER        7
#define FILTER_OUTPUT_CONVOLVE             8

#define BATCH_SIZE 250

/*define errors for condition data and apply search routines here*/

/******** </lalErrTable> */

typedef struct tagSlopeDetections {

  struct tagSlopeDetections* previous;
  UINT4 eventDetectionTimeSeconds;
  UINT4 eventDetectionTimeNanoseconds;
  REAL4 eventAmplitude;
  REAL4 eventDuration;
  REAL4 eventCentralFrequency;
  REAL4 eventBandwidth;
  struct tagSlopeDetections* next;

} SlopeDetections;

typedef struct
tagSlopeSearchParams {

  /* parameters that are part of the SLOPEFilterParams structure */
  /* used by the LAL slopefilter library functions */
  UINT4       forder;
  REAL4*      tap;
  UINT4*      history_allocated;
  UINT4*      taps_set;
  REAL4*      history;
  UINT4       function_select;
  REAL4       waveform_offset;
  REAL4       sampling_period_s;

  /* history buffer for filter at input */
  UINT4       filtHistAllocated;
  UINT4       nSampInFiltHistBuf;
  REAL4*      filtHistBuf;

  /* history buffer for peak search at output */
  UINT4       peakHistAllocated;
  UINT4       nSampInPeakHistBuf;
  REAL4*      peakHistBuf;
  
  /* channel information */
  CHAR        channelName[ maxMultiDimName ]; /*[ 256 ]*/
  CHAR        ifoName[3];
  REAL4       samplingRate;
  UINT4       startTimeSec;
  UINT4       startTimeNan;
  UINT4       stopTimeSec;
  UINT4       stopTimeNan;
  UINT4       nData;
  REAL4*      dataNew;

  REAL4*      filteredData;
  UINT4       dataScanned;
  UINT4       batchSize;
  UINT4       numberOfTriggersInThisSearch;

  /* filter information */
  REAL4       timeIntervalForSlopeMeasure;

  /* trigger information */
  REAL4       triggerThreshold;
  UINT4       numberOfTriggersSoFar;
  UINT4       peakWidth;

  /* MPI information */
  BOOLEAN     mpiMaster;
  INT4        mpiRank;

  /* pointers to first and current event in linked list */
  SlopeDetections* firstEvent;
  SlopeDetections* currentEvent;

  dataBase*   summary;

} SlopeSearchParams;


#endif /* _SLOPEDATATYPESH */



















